/*
 * element.cpp: definition of Element class
 */

#include <math.h>

#include "element.h"

/*
 * construction
 */
Element::Element()
: u(0.0), v(0.0),
  leftElemPtr(0), rightElemPtr(0), aboveElemPtr(0), belowElemPtr(0),
  result(0.0)
{
}

/*
 * add pointers to neighbours (or 0 if no neighbour)
 */
void Element::addNeighbours(Element* _leftElemPtr, Element* _rightElemPtr, Element* _aboveElemPtr, Element* _belowElemPtr)
{
  leftElemPtr  = _leftElemPtr;
  rightElemPtr = _rightElemPtr;
  aboveElemPtr = _aboveElemPtr;
  belowElemPtr = _belowElemPtr;
}

/*
 * set u and v values
 */
void Element::init(double _u, double _v)
{
  u = _u;
  v = _v;
}

/*
 * do some sort of calculation involving the neightbours
 */
void Element::update()
{
  double ul, ur, va, vb;

  ul = ur = va = vb = 0.0;

  // because this is a method of the element class, and we are dealing
  // with pointers to Element objects, we can access their private data
  ul = leftElemPtr ? leftElemPtr->u : 0.0;
  ur = rightElemPtr ? rightElemPtr->u : 0.0;
  va = aboveElemPtr ? aboveElemPtr->v : 0.0;
  vb = belowElemPtr ? belowElemPtr->v : 0.0;

  // 'u' and 'v belong to 'this' instance of the element class
  u = u + pow(2.0, sin(ul*ul - va*va));
  v = v + cos(ur*ur + vb*vb);
}

/*
 * normalise u and v against the length of the vector (u,v)
 */
void Element::normalise()
{
  double m = length();

  // note: no check for divide by zero here
  u /= m;
  v /= m;
}

/*
 * return the length of the vector represented by (u,v)
 */
double Element::length()
{
  return sqrt(u*u + v*v);
}

/*
 * return the sum of u and v
 */
double Element::sumOfParts()
{
  result = u + v;

  return result;
}

/*
 * E N D  O F  F I L E
 */
